// test/main.js
var should = require('should');
var sleepsort = require('../dotformat');

describe('dotformat', function() {
    describe('check to see see if the format function exists in the prototype', function() {
        it('format function exists in the prototype', function() {
			String.prototype.format.should.not.equal(undefined);
        });
    });

	describe('Single argument formatting', function(){
		it('Single string argument works correctly', function(){
			"Hello %s".format("world").should.equal("Hello world");
		});
		it('Single numeric argument works correctly', function(){
			"Hello %s".format(3).should.equal("Hello 3");
		});
		it('Single undefined argument works correctly', function(){
			"Undefined %s".format(undefined).should.equal("Undefined ");
		});
		it('Single null argument works correctly', function(){
			"Hello %s".format(null).should.equal("Hello ");
		});
		it('no argument works correctly', function(){
			"Hello %s".format().should.equal("Hello ");
		});
	});
	
	describe('Test multiple argument formatting', function(){
		it("Multiple string arguments works correctly", function(){
			"Peter %s picked a %s of pickled %s".format("Piper", "peck", "peppers").should.equal("Peter Piper picked a peck of pickled peppers");
		});
		it("Multiple numeric arguments works correctly", function(){
			"Peter %s picked a %s of pickled %s".format(0, 1, 2).should.equal("Peter 0 picked a 1 of pickled 2");
		});
		it("Multiple mixed string and numeric arguments works correctly", function(){
			"Peter %s picked a %s of pickled %s".format("Piper", 1, "peppers").should.equal("Peter Piper picked a 1 of pickled peppers");
		});
		it("Wrong number of arguments works ok", function(){
			"Peter %s picked a %s of pickled %s".format("Piper", 1).should.equal("Peter Piper picked a 1 of pickled ");
		});
		it("undefined with real arguments works ok", function(){
			"Peter %s picked a %s of pickled %s".format("Piper", "peck", undefined).should.equal("Peter Piper picked a peck of pickled ");	
		});
		it("null with real arguments works ok", function(){
			"Peter %s picked a %s of pickled %s".format("Piper", "peck", null).should.equal("Peter Piper picked a peck of pickled ");
		});
	});
	
	describe("Test functional argument formatting", function() {
		it("Functional argument works", function(){
			"Peter %s".format(function(){return "Piper";}).should.equal("Peter Piper");
		});
	});
	
	describe("Test object argument formatting", function() {
		it("Single object argument works correclty", function(){
			"Hello %name%".format({name:"Emmitt"}).should.equal("Hello Emmitt", "Single object argument works correclty");
		});
		it("Single object with multiple arguments works correclty", function(){
			"Hello %name% i am %age% years old".format({name:"Emmitt", age:3}).should.equal("Hello Emmitt i am 3 years old");
		});
		it("Single object out of order params", function(){
			"I am age %age% and my name is %name%".format({name:"Emmitt", age:3}).should.equal("I am age 3 and my name is Emmitt");
		});
	});
	
	describe("Test object argument in a url", function(){
		it("Object as paramater works in url", function(){
			"http://www.google.com/q=%search%".format({search:"Hello+World"}).should.equal("http://www.google.com/q=Hello+World");
		});
		it("Object with multiple entries as paramater works in url", function(){
			"%protocol%//www.%domain%.com%path%".format({protocol:"https:", domain:"locationinsight", path:"/hello/world"}).should.equal("https://www.locationinsight.com/hello/world");
		});
	});
	
	describe("Test mutilple arguments in a url", function(){
		it("Single argument works in url", function(){
			"http://www.google.com/q=%s".format("Hello+World").should.equal("http://www.google.com/q=Hello+World");
		});
		it("Multiple arguments work in url", function(){
			"%s//www.%s.com%s".format("https:", "locationinsight", "/hello/world").should.equal("https://www.locationinsight.com/hello/world");
		});
	});
	
	describe("Test single array argument formatting", function() {
		it("Single string in array argument works correctly", function(){
			"Hello %s".format(["world"]).should.equal("Hello world");
		});
		it("Single numeric in array argument works correctly", function(){
			"Hello %s".format([3]).should.equal("Hello 3");
		});
		it("Single undefined in array argument works correctly", function(){
			"Undefined %s".format([undefined]).should.equal("Undefined ");
		});
		it("Single null in array argument works correctly", function(){
			"Hello %s".format([null]).should.equal("Hello ");
		});
		it("empty array works correctly", function(){
			"Hello %s".format([]).should.equal("Hello ");
		});
	});
	
	describe("Test single array argument formatting", function() {
		it("Multiple string arguments in array works correctly", function(){
			"Peter %s picked a %s of pickled %s".format(["Piper", "peck", "peppers"]).should.equal("Peter Piper picked a peck of pickled peppers");
		});
		it("Multiple numeric arguments in array works correctly", function(){
			"Peter %s picked a %s of pickled %s".format([0, 1, 2]).should.equal("Peter 0 picked a 1 of pickled 2");
		});
		it("Multiple mixed string and numeric arguments in array works correctly", function(){
			"Peter %s picked a %s of pickled %s".format(["Piper", 1, "peppers"]).should.equal("Peter Piper picked a 1 of pickled peppers");
		});
		it("Wrong number of arguments in array works ok", function(){
			"Peter %s picked a %s of pickled %s".format(["Piper", 1]).should.equal("Peter Piper picked a 1 of pickled ");
		});
		it("undefined with real arguments in array works ok", function(){
			"Peter %s picked a %s of pickled %s".format(["Piper", "peck", undefined]).should.equal("Peter Piper picked a peck of pickled ");
		});
		it("null with real arguments in array works ok", function(){
			"Peter %s picked a %s of pickled %s".format(["Piper", "peck", null]).should.equal("Peter Piper picked a peck of pickled ");
		});
	});
	
	describe("Replace regex matcher", function() {
		it("Changing out delimeter of single arguments to ':\d{1}s'", function(){
			"Peter :1s picked a :2s of pickled :3s".format("Piper", "peck", "peppers", /\:[\d]s/g).should.equal("Peter Piper picked a peck of pickled peppers");
		});
		it("Single object argument delimeter switching to ':[^:]*:'", function(){
			"Hello :name:".format({name:"Emmitt"}, new RegExp("\:([^\:]*)\:", "g") ).should.equal("Hello Emmitt", "Single object argument works correclty");
		});
		it("Object as paramater with replacement regext works in url", function(){
			"http://www.google.com/?q=<search>&foo=<baz>".format({search:"Hello+World",baz:"clover"}, /\<([^\>]*)\>/g).should.equal("http://www.google.com/?q=Hello+World&foo=clover");
		});
		it("Multi-character delimiters", function(){
			"Somethign I do is {{something}}".format({something:"EAT MOTHER EFFING CHILI"}, /\{\{(.+?)\}\}/g).should.equal("Somethign I do is EAT MOTHER EFFING CHILI");
		});
	});
});